package com.jkcredit.traffic.record.action;

import com.ciphergateway.ciphersuite.CipherSuiteException;
import com.ciphergateway.ciphersuite.CipherSuiteMacException;
import com.ciphergateway.ciphersuite.CipherSuiteUtils;
import com.jkcredit.traffic.record.model.CommonResponseObject;
import com.jkcredit.traffic.record.service.TrafficRecordsAnalyseService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.xml.bind.DatatypeConverter;
import java.io.UnsupportedEncodingException;

/**
 * @author xusonglin
 * @version V1.0
 **/
@RestController
@Slf4j
@RequestMapping("/api")
public class TrafficRecordsAction {
    @Autowired
    TrafficRecordsAnalyseService service;

    @PostMapping("/push")
    public CommonResponseObject pushTrafficRecords(@RequestParam String path) {
        return service.pushTrafficRecords(path);
    }

    @PostMapping("/pushErrorRecords")
    public CommonResponseObject pushErrorRecords(@RequestParam String path) {
        return service.pushErrorRecords(path);
    }

    @PostMapping("/encrypt")
    public String vehicleIdEncrypt(@RequestParam String vehicleId, @RequestParam String algorithm,
                                    @RequestParam String keyId, @RequestParam String metadata, @RequestParam String ivStr) {
        try {
            byte[] plainData = vehicleId.getBytes("UTF-8");
            byte[] iv = ivStr.getBytes("UTF-8");
            Long startTime = System.currentTimeMillis();
            // 加密
            byte[] cipherData = CipherSuiteUtils.encrypt(plainData, algorithm, keyId, metadata, iv);
            return DatatypeConverter.printHexBinary(cipherData);
        } catch (UnsupportedEncodingException ue) {
            ue.printStackTrace();
            log.error("UnsupportedEncodingException:", ue);
            return "";
        } catch (CipherSuiteException cse) {
            cse.printStackTrace();
            log.error("CipherSuiteException:", cse);
            return "";
        } catch (CipherSuiteMacException csme) {
            csme.printStackTrace();
            log.error("CipherSuiteMacException:", csme);
            return "";
        } catch (Exception e) {
            e.printStackTrace();
            log.error("Exception:", e);
            return "";
        } catch (Error error) {
            error.printStackTrace();
            log.error("Error:", error);
            return "";
        }
    }
}
